# -*- coding: utf-8 -*-
"""Weather forecast Python model for the Météo-France REST API."""
import sys
from datetime import datetime
from typing import Any
from typing import Dict
from typing import List

from pytz import utc

from meteofrance_api.helpers import timestamp_to_dateime_with_locale_tz

if sys.version_info >= (3, 8):
    from typing import TypedDict  # pylint: disable=no-name-in-module
else:
    from typing_extensions import TypedDict


class ForecastData(TypedDict, total=False):
    """Describing the data structure of the forecast object returned by the REST API."""

    position: Dict[str, Any]
    updated_on: int
    daily_forecast: List[Dict[str, Any]]
    forecast: List[Dict[str, Any]]
    probability_forecast: List[Dict[str, Any]]


class Forecast:
    """Class to access the results of a `forecast` API request.

    Attributes:
        position: A dictionary with metadata about the position of the forecast place.
        updated_on: A timestamp as int corresponding to the latest update date.
        daily_forecast: A list of dictionaries to describe the daily forecast for the
            next 15 days.
        forecast: A list of dictionaries to describe the hourly forecast for the next
            days.
        probability_forecast: A list of dictionaries to describe the event probability
            forecast (rain, snow, freezing) for next 10 days.
        today_forecast: A dictionary corresponding to the daily forecast for the current
        day.
        nearest_forecast: A dictionary corresponding to the nearest hourly forecast.
        current_forecast: A dictionary corresponding to the hourly forecast for the
            current hour.
    """

    def __init__(self, raw_data: ForecastData) -> None:
        """Initialize a Forecast object.

        Args:
            raw_data: A dictionary representing the JSON response from 'forecast' REST
                API request. The structure is described by the ForecastData class.
        """
        self.raw_data = raw_data

    @property
    def position(self) -> Dict[str, Any]:
        """Return the position information of the forecast."""
        return self.raw_data["position"]

    @property
    def updated_on(self) -> int:
        """Return the update timestamp of the forecast."""
        return self.raw_data["updated_on"]

    @property
    def daily_forecast(self) -> List[Dict[str, Any]]:
        """Return the daily forecast for the following days."""
        return self.raw_data["daily_forecast"]

    @property
    def forecast(self) -> List[Dict[str, Any]]:
        """Return the hourly forecast."""
        return self.raw_data["forecast"]

    @property
    def probability_forecast(self) -> List[Dict[str, Any]]:
        """Return the wheather event forecast."""
        return self.raw_data.get("probability_forecast", [])

    @property
    def today_forecast(self) -> Dict[str, Any]:
        """Return the forecast for today."""
        return self.daily_forecast[0]

    @property
    def nearest_forecast(self) -> Dict[str, Any]:
        """Return the nearest hourly forecast."""
        # get timestamp for current time
        now_timestamp = int(utc.localize(datetime.utcnow()).timestamp())
        # sort list of forecast by distance between current timestamp and
        # forecast timestamp
        sorted_forecast = sorted(
            self.forecast,
            key=lambda x: abs(x["dt"] - now_timestamp),  # type: ignore[no-any-return]
        )
        return sorted_forecast[0]

    @property
    def current_forecast(self) -> Dict[str, Any]:
        """Return the forecast of the current hour."""
        # Get the timestamp for the current hour.
        current_hour_timestamp = int(
            utc.localize(
                datetime.utcnow().replace(minute=0, second=0, microsecond=0)
            ).timestamp()
        )
        # create a dict using timestamp as keys
        forecast_by_datetime = {item["dt"]: item for item in self.forecast}
        # Return the forecast corresponding to the timestamp of the current hour if
        # exists. If not exists, returns the nearest forecast (not France countries)
        return forecast_by_datetime.get(current_hour_timestamp, self.nearest_forecast)

    def timestamp_to_locale_time(self, timestamp: int) -> datetime:
        """Convert timestamp in datetime in the forecast location timezone (Helper).

        Args:
            timestamp: An integer to describe the UNIX timestamp.

        Returns:
            Datetime instance corresponding to the timestamp with the timezone of the
                forecast location.
        """
        return timestamp_to_dateime_with_locale_tz(timestamp, self.position["timezone"])



# -*- coding: utf-8 -*-
"""Session managers for the Météo-France REST API."""
from typing import Any
from typing import Optional

from requests import Response
from requests import Session

from .const import METEOFRANCE_API_TOKEN
from .const import METEOFRANCE_API_URL
from .const import METEOFRANCE_WS_API_URL
from .const import METEONET_API_URL


class MeteoFranceSession(Session):
    """HTTP session manager for Météo-France.

    This session object allows to manage the authentication in the API using a token.
    """

    host: str = METEOFRANCE_API_URL

    def __init__(self, access_token: Optional[str] = None) -> None:
        """Initialize the authentication.

        Args:
            access_token: a string containing the authentication token for the REST API.
        """
        self.access_token = access_token or METEOFRANCE_API_TOKEN
        Session.__init__(self)

    def request(  # type: ignore
        self, method: str, path: str, *args: Any, **kwargs: Any
    ) -> Response:
        """Make a request using token authentication.

        Args:
            method: Method for the HTTP request (example "get").
            path: path of the REST API endpoint.
            args: all other non-keyword arguments.
            kwargs: all other keyword arguments.

        Returns:
            the Response object corresponding to the result of the API request.
        """
        params_inputs = kwargs.pop("params", None)

        params = {"token": self.access_token}
        if params_inputs:
            params.update(params_inputs)

        kwargs["params"] = params
        response = super().request(method, f"{self.host}/{path}", *args, **kwargs)
        response.raise_for_status()

        return response


class MeteoFranceWSSession(MeteoFranceSession):
    """HTTP session manager for Météo-France WS.

    Child class dedicated to additional REST API URI for specific command like getDetail
    , getAllVigilances, getVigilance.
    """

    host: str = METEOFRANCE_WS_API_URL

    def __init__(self, access_token: Optional[str] = None) -> None:
        """Initialize the Météo-France WS.

        Args:
            access_token: a string containing the authentication token for the REST API.
        """
        super().__init__(access_token)


class MeteoNetSession(MeteoFranceSession):
    """HTTP session manager for MétéoNet.

    Child class dedicated to additional REST API URI for specific command like ImageJour
    , radarEU.
    """

    host: str = METEONET_API_URL

    def __init__(self, access_token: Optional[str] = None) -> None:
        """Initialize the MétéoNet.

        Args:
            access_token: a string containing the authentication token for the REST API.
        """
        super().__init__(access_token)



# -*- coding: utf-8 -*-
"""Client for the Météo-France REST API."""
from typing import List
from typing import Optional

from .const import COASTAL_DEPARTMENT_LIST
from .const import METEOFRANCE_API_TOKEN
from .const import METEOFRANCE_API_URL
from .model import CurrentPhenomenons
from .model import Forecast
from .model import Full
from .model import PictureOfTheDay
from .model import Place
from .model import Rain
from .session import MeteoFranceSession
from .session import MeteoFranceWSSession
from .session import MeteoNetSession

# TODO: http://webservice.meteofrance.com/observation
# TODO: investigate bulletincote, montagne, etc...
#       http://ws.meteofrance.com/ws//getDetail/france/330630.json
# TODO: add protection for warning if domain not valid
# TODO: strategy for HTTP errors
# TODO: next rain in minute. Necessary ?
# TODO: forecast/metadata from ID to get gps ?


class MeteoFranceClient:
    """Proxy to the Météo-France REST API.

    You will find methods and helpers to request weather forecast, rain forecast and
    weather alert bulletin.
    """

    def __init__(self, access_token: Optional[str] = None) -> None:
        """Initialize the API and store the auth so we can make requests.

        Args:
            access_token: a string containing the authentication token for the REST API.
        """
        self.session = MeteoFranceSession(access_token)
        self.session_ws = MeteoFranceWSSession()
        self.session_net = MeteoNetSession()

    #
    # Place
    #
    def search_places(
        self,
        search_query: str,
        latitude: Optional[str] = None,
        longitude: Optional[str] = None,
    ) -> List[Place]:
        """Search the places (cities) linked to a query by name.

        You can add GPS coordinates in parameter to search places arround a given
        location.

        Args:
            search_query: A complete name, only a part of a name or a postal code (for
                France only) corresponding to a city in the world.
            latitude: Optional; Latitude in degree of a reference point to order
                results. The nearest places first.
            longitude: Optional; Longitude in degree of a reference point to order
                results. The nearest places first.

        Returns:
            A list of places (Place instance) corresponding to the query.
        """
        # Construct the list of the GET parameters
        params = {"q": search_query}
        if latitude is not None:
            params["lat"] = latitude
        if longitude is not None:
            params["lon"] = longitude

        # Send the API resuest
        resp = self.session.request("get", "places", params=params)
        return [Place(place_data) for place_data in resp.json()]

    #
    # Forecast
    #
    def get_forecast(
        self,
        latitude: float,
        longitude: float,
        language: str = "fr",
    ) -> Forecast:
        """Retrieve the weather forecast for a given GPS location.

        Results can be fetched in french or english according to the language parameter.

        Args:
            latitude: Latitude in degree of the GPS point corresponding to the weather
                forecast.
            longitude: Longitude in degree of the GPS point corresponding to the weather
                forecast.
            language: Optional; If language is equal "fr" (default value) results will
                be in French. All other value will give results in English.

        Returns:
            A Forecast intance representing the hourly and daily weather forecast.
        """
        # TODO: add possibility to request forecast from id

        # Send the API request
        resp = self.session.request(
            "get",
            "forecast",
            params={"lat": latitude, "lon": longitude, "lang": language},
        )
        return Forecast(resp.json())

    def get_forecast_for_place(
        self,
        place: Place,
        language: str = "fr",
    ) -> Forecast:
        """Retrieve the weather forecast for a given Place instance.

        Results can be fetched in french or english according to the language parameter.

        Args:
            place: Place class instance corresponding to a location.
            language: Optional; If language is equal "fr" (default value) results will
                be in French. All other value will give results in English.

        Returns:
            A Forecast intance representing the hourly and daily weather forecast.
        """
        return self.get_forecast(place.latitude, place.longitude, language)

    #
    # Rain
    #
    def get_rain(self, latitude: float, longitude: float, language: str = "fr") -> Rain:
        """Retrieve the next 1 hour rain forecast for a given GPS the location.

        Results can be fetched in french or english according to the language parameter.

        Args:
            latitude: Latitude in degree of the GPS point corresponding to the rain
                forecast.
            longitude: Longitude in degree of the GPS point corresponding to the rain
                forecast.
            language: Optional; If language is equal "fr" (default value) results will
                be in French. All other value will give results in English.

        Returns:
            A Rain instance representing the next hour rain forecast.
        """
        # TODO: add protection if no rain forecast for this position

        # Send the API request
        resp = self.session.request(
            "get", "rain", params={"lat": latitude, "lon": longitude, "lang": language}
        )
        return Rain(resp.json())

    #
    # Warning
    #
    def get_warning_current_phenomenoms(
        self, domain: str, depth: int = 0, with_costal_bulletin: bool = False
    ) -> CurrentPhenomenons:
        """Return the current weather phenomenoms (or alerts) for a given domain.

        Args:
            domain: could be `france` or any metropolitan France department numbers on
                two digits. For some departments you can access an additional bulletin
                for coastal phenomenoms. To access it add `10` after the domain id
                (example: `1310`).
            depth: Optional; To be used with domain = 'france'. With depth = 0 the
                results will show only natinal sum up of the weather alerts. If
                depth = 1, you will have in addition, the bulletin for all metropolitan
                France department and Andorre
            with_costal_bulletin: Optional; If set to True (default is False), you can
                get the basic bulletin and coastal bulletin merged.

        Returns:
            A warning.CurrentPhenomenons instance representing the weather alert
            bulletin.
        """
        # Send the API request
        resp = self.session.request(
            "get",
            "warning/currentphenomenons",
            params={"domain": domain, "depth": depth},
        )

        # Create object with API response
        phenomenoms = CurrentPhenomenons(resp.json())

        # if user ask to have the coastal bulletin merged
        if with_costal_bulletin:
            if domain in COASTAL_DEPARTMENT_LIST:
                resp = self.session.request(
                    "get",
                    "warning/currentphenomenons",
                    params={"domain": domain + "10"},
                )
                phenomenoms.merge_with_coastal_phenomenons(
                    CurrentPhenomenons(resp.json())
                )

        return phenomenoms

    def get_warning_full(self, domain: str, with_costal_bulletin: bool = False) -> Full:
        """Retrieve a complete bulletin of the weather phenomenons for a given domain.

        For a given domain we can access the maximum alert, a timelaps of the alert
        evolution for the next 24 hours, a list of alerts and other metadatas.

        Args:
            domain: could be `france` or any metropolitan France department numbers on
                two digits. For some departments you can access an additional bulletin
                for coastal phenomenoms. To access it add `10` after the domain id
                (example: `1310`).
            with_costal_bulletin: Optional; If set to True (default is False), you can
                get the basic bulletin and coastal bulletin merged.

        Returns:
            A warning.Full instance representing the complete weather alert bulletin.
        """
        # TODO: add formatDate parameter

        # Send the API request
        resp = self.session.request("get", "warning/full", params={"domain": domain})

        # Create object with API response
        full_phenomenoms = Full(resp.json())

        # if user ask to have the coastal bulletin merged
        if with_costal_bulletin:
            if domain in COASTAL_DEPARTMENT_LIST:
                resp = self.session.request(
                    "get",
                    "warning/full",
                    params={"domain": domain + "10"},
                )
                full_phenomenoms.merge_with_coastal_phenomenons(Full(resp.json()))

        return full_phenomenoms

    def get_warning_thumbnail(self, domain: str = "france") -> str:
        """Retrieve the thumbnail URL of the weather phenomenoms or alerts map.

        Args:
            domain: could be `france` or any metropolitan France department numbers on
                two digits.

        Returns:
            The URL of the thumbnail representing the weather alert status.
        """
        # Return directly the URL of the gif image
        return (
            f"{METEOFRANCE_API_URL}/warning/thumbnail?&token={METEOFRANCE_API_TOKEN}"
            f"&domain={domain}"
        )

    #
    # Picture of the day
    #
    def get_picture_of_the_day(self) -> PictureOfTheDay:
        """Retrieve the picture of the day image URL & description.

        Returns:
            PictureOfTheDay instance with the URL and the description of the picture of
            the day.
        """
        resp = self.session_net.request("get", "ImageJour/last.txt")
        return PictureOfTheDay(resp.json())



# -*- coding: utf-8 -*-
"""Weather alert buletin Python model for the Météo-France REST API.

For getting weather alerts in France Metropole and Andorre.
"""
import sys
from typing import Any
from typing import Dict
from typing import List
from typing import Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict  # pylint: disable=no-name-in-module
else:
    from typing_extensions import TypedDict


class WarnningCurrentPhenomenonsData(TypedDict):
    """Describing the data structure of CurrentPhenomenons object from the REST API."""

    update_time: int
    end_validity_time: int
    domain_id: str
    phenomenons_max_colors: List[Dict[str, int]]


class WarnningFullData(TypedDict):
    """Describing the data structure of full object from the REST API."""

    update_time: int
    end_validity_time: int
    domain_id: str
    color_max: int
    timelaps: List[Dict[str, Any]]
    phenomenons_items: List[Dict[str, int]]
    advices: Optional[List[Dict[str, Any]]]
    consequences: Optional[List[Dict[str, Any]]]
    max_count_items: Any  # Didn't see any value yet
    comments: Dict[str, Any]
    text: Optional[Dict[str, Any]]
    text_avalanche: Any  # Didn't see any value yet


class CurrentPhenomenons:
    """Class to access the results of a `warning/currentPhenomenons` REST API request.

    For coastal department two bulletins are avalaible corresponding to two different
    domains.

    Attributes:
        update_time: A timestamp (as integer) corresponding to the latest update of the
            pheomenoms.
        end_validity_time: A timestamp (as integer) corresponding to expiration date of
            the phenomenoms.
        domain_id: A string corresponding do the domain ID of the bulletin. Value is
            'France' or a department number.
        phenomenons_max_colors: A list of dictionnaries with type of phenomenoms and the
            current alert level.
    """

    def __init__(self, raw_data: WarnningCurrentPhenomenonsData) -> None:
        """Initialize a CurrentPhenomenons object.

        Args:
            raw_data: A dictionary representing the JSON response from
                'warning/currentPhenomenons' REST API request. The structure is
                described by the WarnningCurrentPhenomenonsData class.
        """
        self.raw_data = raw_data

    @property
    def update_time(self) -> int:
        """Return the update time of the phenomenoms."""
        return self.raw_data["update_time"]

    @property
    def end_validity_time(self) -> int:
        """Return the end of validty time of the phenomenoms."""
        return self.raw_data["end_validity_time"]

    @property
    def domain_id(self) -> str:
        """Return the domain ID of the phenomenoms."""
        return self.raw_data["domain_id"]

    @property
    def phenomenons_max_colors(self) -> List[Dict[str, int]]:
        """Return the list and colors of the phenomenoms."""
        return self.raw_data["phenomenons_max_colors"]

    def merge_with_coastal_phenomenons(
        self, coastal_phenomenoms: "CurrentPhenomenons"
    ) -> None:
        """Merge the classical phenomenoms bulleting with the coastal one.

        Extend the phenomenomes_max_colors property with the content of the coastal
        weather alert bulletin.

        Args:
            coastal_phenomenoms: CurrentPhenomenons instance corresponding to the
                coastal weather alert bulletin.
        """
        # TODO: Add consitency check
        self.raw_data["phenomenons_max_colors"].extend(
            coastal_phenomenoms.phenomenons_max_colors
        )

    def get_domain_max_color(self) -> int:
        """Get the maximum level of alert of a given domain (class helper).

        Returns:
            An integer corresponding to the status code representing the maximum alert.
        """
        max_int_color = max(
            x["phenomenon_max_color_id"] for x in self.phenomenons_max_colors
        )
        return max_int_color


class Full:
    """This class allows to access the results of a `warning/full` API command.

    For a given domain we can access the maximum alert, a timelaps of the alert
    evolution for the next 24 hours, and a list of alerts.

    For coastal department two bulletins are avalaible corresponding to two different
    domains.

    Attributes:
        update_time: A timestamp (as integer) corresponding to the latest update of the
            pheomenoms.
        end_validity_time: A timestamp (as integer) corresponding to expiration date of
            the phenomenoms.
        domain_id: A string corresponding do the domain ID of the bulletin. Value is
            'France' or a department number.
        color_max: An integer representing the maximum alert level in the domain.
        timelaps: A list of dictionnaries corresponding to the schedule of each
            phenomenoms in the next 24 hours.
        phenomenons_items: list of dictionnaries corresponding the alert level for each
            phenomenoms type.
    """

    def __init__(self, raw_data: WarnningFullData) -> None:
        """Initialize a Full object.

        Args:
            raw_data: A dictionary representing the JSON response from'warning/full'
                REST API request. The structure is described by the WarnningFullData
                class.
        """
        self.raw_data = raw_data

    @property
    def update_time(self) -> int:
        """Return the update time of the full bulletin."""
        return self.raw_data["update_time"]

    @property
    def end_validity_time(self) -> int:
        """Return the end of validty time of the full bulletin."""
        return self.raw_data["end_validity_time"]

    @property
    def domain_id(self) -> str:
        """Return the domain ID of the the full bulletin."""
        return self.raw_data["domain_id"]

    @property
    def color_max(self) -> int:
        """Return the color max of the domain."""
        return self.raw_data["color_max"]

    @property
    def timelaps(self) -> List[Dict[str, Any]]:
        """Return the timelaps of each phenomenom for the domain."""
        return self.raw_data["timelaps"]

    @property
    def phenomenons_items(self) -> List[Dict[str, int]]:
        """Return the phenomenom list of the domain."""
        return self.raw_data["phenomenons_items"]

    def merge_with_coastal_phenomenons(self, coastal_phenomenoms: "Full") -> None:
        """Merge the classical phenomenon bulletin with the coastal one.

        Extend the color_max, timelaps and phenomenons_items properties with the content
            of the coastal weather alert bulletin.

        Args:
            coastal_phenomenoms: Full instance corresponding to the coastal weather
                alert bulletin.
        """
        # TODO: Add consitency check
        # TODO: Check if other data need to be merged

        # Merge color_max property
        self.raw_data["color_max"] = max(self.color_max, coastal_phenomenoms.color_max)

        # Merge timelaps
        self.raw_data["timelaps"].extend(coastal_phenomenoms.timelaps)

        # Merge phenomenons_items
        self.raw_data["phenomenons_items"].extend(coastal_phenomenoms.phenomenons_items)

    # TODO: check opportunity to complete class



# -*- coding: utf-8 -*-
"""Consts for Météo-France weather forecast python API."""
METEOFRANCE_API_URL = "http://webservice.meteofrance.com"
# Additionan API URL used with getDetail, getAllVigilances, getVigilance
METEOFRANCE_WS_API_URL = "http://ws.meteofrance.com/ws"
# Additionan API URL used with ImageJour, radarEU
METEONET_API_URL = "http://www.meteo.fr/meteonet/temps"
METEOFRANCE_API_TOKEN = "__Wj7dVSTjV9YGu1guveLyDq0g7S7TfTjaHBTPTpO0kj8__"

# enums used in all Warning classes. First indice is 0
# Weather alert criticity
ALERT_COLOR_LIST_FR = [None, "Vert", "Jaune", "Orange", "Rouge"]
ALERT_COLOR_LIST_EN = [None, "Green", "Yellow", "Orange", "Red"]

# Weather alert type
ALERT_TYPE_LIST_FR = [
    None,
    "Vent violent",
    "Pluie-inondation",
    "Orages",
    "Inondation",
    "Neige-verglas",
    "Canicule",
    "Grand-froid",
    "Avalanches",
    "Vagues-submersion",
]

ALERT_TYPE_LIST_EN = [
    None,
    "Wind",
    "Rain-Flood",
    "Thunderstorms",
    "Flood",
    "Snow/Ice",
    "Extreme high temperature",
    "Extreme low temperature",
    "Avalanches",
    "Coastal Event",
]


# Valide departments list for weather alert bulletin
VALID_DEPARTMENT_LIST = [
    "01",
    "02",
    "03",
    "04",
    "05",
    "06",
    "07",
    "08",
    "09",
    "10",
    "11",
    "12",
    "13",
    "14",
    "15",
    "16",
    "17",
    "18",
    "19",
    "20",
    "21",
    "22",
    "23",
    "24",
    "25",
    "26",
    "27",
    "28",
    "29",
    "2A",
    "2B",
    "30",
    "31",
    "32",
    "33",
    "34",
    "35",
    "36",
    "37",
    "38",
    "39",
    "40",
    "41",
    "42",
    "43",
    "44",
    "45",
    "46",
    "47",
    "48",
    "49",
    "50",
    "51",
    "52",
    "53",
    "54",
    "55",
    "56",
    "57",
    "58",
    "59",
    "60",
    "61",
    "62",
    "63",
    "64",
    "65",
    "66",
    "67",
    "68",
    "69",
    "70",
    "71",
    "72",
    "73",
    "74",
    "75",
    "76",
    "77",
    "78",
    "79",
    "80",
    "81",
    "82",
    "83",
    "84",
    "85",
    "86",
    "87",
    "88",
    "89",
    "90",
    "91",
    "92",
    "93",
    "94",
    "95",
    "99",
]

# Area code list for Coastal Departments
COASTAL_DEPARTMENT_LIST = [
    "06",
    "11",
    "13",
    "14",
    "17",
    "22",
    "29",
    "2A",
    "2B",
    "30",
    "33",
    "34",
    "35",
    "40",
    "44",
    "50",
    "56",
    "59",
    "62",
    "64",
    "66",
    "76",
    "80",
    "83",
    "85",
]



# -*- coding: utf-8 -*-
"""Picture of the Day Python model for the Météo-France REST API."""
import sys
from typing import Mapping

if sys.version_info >= (3, 8):
    from typing import TypedDict  # pylint: disable=no-name-in-module
else:
    from typing_extensions import TypedDict


class PictureOfTheDayData(TypedDict):
    """Describing the data structure of ImageJour object returned by the REST API."""

    vignette: str
    imageHD: str  # noqa: N815
    commentaire: str


class PictureOfTheDay:
    """Class to access the results of a `ImageJour/last` REST API request.

    Attributes:
        image_url: A string corresponding to the picture of the day URL.
        image_hd_url: A string corresponding to the URL for the HD version of the
            picture of the day.
        descritpion: A string with the description of the picture of the day.
    """

    def __init__(self, raw_data: Mapping[str, PictureOfTheDayData]) -> None:
        """Initialize a PictureOfTheDay object.

        Args:
            raw_data: A dictionary representing the JSON response from 'ImageJour/last'
                REST API request. The structure is described by the PictureOfTheDayData
                class.
        """
        self.raw_data = raw_data["result"]

    @property
    def image_url(self) -> str:
        """Return the image URL of the picture of the day."""
        return self.raw_data["vignette"]

    @property
    def image_hd_url(self) -> str:
        """Return the image HD URL of the picture of the day."""
        return self.raw_data["imageHD"]

    @property
    def description(self) -> str:
        """Return the description of the picture of the day."""
        return self.raw_data["commentaire"]



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['meteofrance_api', 'meteofrance_api.model']

package_data = \
{'': ['*']}

install_requires = \
['pytz>=2020.1,<2021.0', 'requests>=2.24.0,<3.0.0']

extras_require = \
{':python_version >= "3.6" and python_version < "3.8"': ['typing-extensions>=3.7.4,<4.0.0']}

entry_points = \
{'console_scripts': ['meteofrance-api = meteofrance_api.__main__:main']}

setup_kwargs = {
    'name': 'meteofrance-api',
    'version': '1.0.0',
    'description': 'Python client for Météo-France API.',
    'long_description': "Météo-France Python API\n=======================\n\nClient Python pour l'API Météo-France. | Python client for Météo-France API.\n\n|PyPI| |GitHub Release| |Python Version| |License|\n\n|Read the Docs| |Tests| |Codecov| |GitHub Activity|\n\n|pre-commit| |Black|\n\n\n.. |PyPI| image:: https://img.shields.io/pypi/v/meteofrance-api\n   :target: https://pypi.org/project/meteofrance-api/\n   :alt: PyPI\n.. |GitHub Release| image:: https://img.shields.io/github/release/hacf-fr/meteofrance-api.svg\n   :target: https://github.com/hacf-fr/meteofrance-api/releases\n   :alt: GitHub Release\n.. |Python Version| image:: https://img.shields.io/pypi/pyversions/meteofrance-api\n   :target: https://pypi.org/project/meteofrance-api/\n   :alt: Python Version\n.. |License| image:: https://img.shields.io/pypi/l/meteofrance-api\n   :target: https://opensource.org/licenses/MIT\n   :alt: License\n.. |Read the Docs| image:: https://img.shields.io/readthedocs/meteofrance-api/latest.svg?label=Read%20the%20Docs\n   :target: https://meteofrance-api.readthedocs.io/\n   :alt: Read the documentation at https://meteofrance-api.readthedocs.io/\n.. |Tests| image:: https://github.com/hacf-fr/meteofrance-api/workflows/Tests/badge.svg\n   :target: https://github.com/hacf-fr/meteofrance-api/actions?workflow=Tests\n   :alt: Tests\n.. |Codecov| image:: https://codecov.io/gh/hacf-fr/meteofrance-api/branch/master/graph/badge.svg\n   :target: https://codecov.io/gh/hacf-fr/meteofrance-api\n   :alt: Codecov\n.. |GitHub Activity| image:: https://img.shields.io/github/commit-activity/y/hacf-fr/meteofrance-api.svg\n   :target: https://github.com/hacf-fr/meteofrance-api/commits/master\n   :alt: GitHub Activity\n.. |pre-commit| image:: https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white\n   :target: https://github.com/pre-commit/pre-commit\n   :alt: pre-commit\n.. |Black| image:: https://img.shields.io/badge/code%20style-black-000000.svg\n   :target: https://github.com/psf/black\n   :alt: Black\n\nYou will find English README content in the section `For English speaking users`_.\n\nVous trouverez le contenu francophone du README dans la section `Pour les francophones`_.\n\nPour les francophones\n---------------------\n\nDescription\n^^^^^^^^^^^\n\nCe package Python permet de gérer la communication avec l'API non publique de\nMétéo-France utilisée par les applications mobiles officielles.\n\nLe client permet:\n\n* Rechercher des lieux de prévisions.\n* Accéder aux prévisions météorologiques horaires ou quotidiennes.\n* Accéder aux prévisions de pluie dans l'heure quand disponibles.\n* Accéder aux alertes météo pour chaque département français et d'Andorre. Deux\n  bulletins sont disponibles : un synthétique et un second avec l'évolution des alertes\n  pour les prochaines 24 heures (exemple `ici <https://vigilance.meteofrance.fr/fr/gers>`_).\n\nCe package a été développé avec l'intention d'être utilisé par `Home-Assistant <https://home-assistant.io/>`_\nmais il peut être utilsé dans d'autres contextes.\n\nInstallation\n^^^^^^^^^^^^\n\nPour utiliser le module Python ``meteofrance`` vous devez en premier installer\nle package en utilisant pip_ depuis PyPI_:\n\n.. code:: console\n\n   $ pip install meteofrance-api\n\n\nVous pouvez trouver un exemple d'usage dans un module Python en regardant\n`le test d'intégration <tests/test_integrations.py>`_.\n\nContribuer\n^^^^^^^^^^\n\nLes contributions sont les bienvenues. Veuillez consulter les bonnes pratiques\ndétaillées dans `CONTRIBUTING.rst`_.\n\n\nFor English speaking users\n--------------------------\n\nDescrtption\n^^^^^^^^^^^^\n\nThis Python package manages the communication with the private Météo-France API\nused by the official mobile applications.\n\nThe client allows:\n\n* Search a forecast location.\n* Fetch daily or hourly weather forecast.\n* Fetch rain forecast within the next hour if available.\n* Fetch the weather alerts or phenomenoms for each French department or Andorre.\n  Two bulletin are availabe: one basic and an other advanced with the timelaps evolution\n  for the next 24 hours (example `here <https://vigilance.meteofrance.fr/fr/gers>`_).\n\nThis package have been developed to be used with `Home-Assistant <https://home-assistant.io/>`_\nbut it can be used in other contexts.\n\nInstallation\n^^^^^^^^^^^^\n\nTo use the ``meteofrance`` Python module, you have to install this package first via\npip_ from PyPI_:\n\n.. code:: console\n\n   $ pip install meteofrance-api\n\nYou will find an example ot usage in a Python program in the `integration test <tests/test_integrations.py>`_.\n\nContributing\n^^^^^^^^^^^^\n\nContributions are welcomed. Please check the guidelines in `CONTRIBUTING.rst`_.\n\nCredits\n-------\n\nThis project was generated from `@cjolowicz`_'s `Hypermodern Python Cookiecutter`_ template.\n\n.. _@cjolowicz: https://github.com/cjolowicz\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _PyPI: https://pypi.org/\n.. _Hypermodern Python Cookiecutter: https://github.com/cjolowicz/cookiecutter-hypermodern-python\n.. _pip: https://pip.pypa.io/\n\n.. github-only\n.. _CONTRIBUTING.rst: CONTRIBUTING.rst\n",
    'author': 'oncleben31',
    'author_email': 'oncleben31@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/hacf-fr/meteofrance-api',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



"""Météo-France models for the REST API."""
from .forecast import Forecast
from .picture_of_the_day import PictureOfTheDay
from .place import Place
from .rain import Rain
from .warning import CurrentPhenomenons
from .warning import Full

__all__ = ["Forecast", "Place", "PictureOfTheDay", "Rain", "CurrentPhenomenons", "Full"]



# -*- coding: utf-8 -*-
"""Place Python model for the Météo-France REST API."""
import sys
from typing import Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict  # pylint: disable=no-name-in-module
else:
    from typing_extensions import TypedDict


class PlaceData(TypedDict):
    """Describing the data structure of place object returned by the REST API."""

    insee: Optional[str]
    name: str
    lat: float
    lon: float
    country: str
    admin: str
    admin2: Optional[str]
    postCode: Optional[str]  # noqa: N815


class Place:
    """Class to access the results of 'places' REST API request.

    Attributes:
        insee: A string corresponding to the INSEE ID of the place.
        name: Name of the place.
        lat: A float with the latitude in degree of the place.
        lon: A float with the longitude in degree of the place
        country: A string corresponding to the country code of the place.
        admin: A string with the name of the administrative area ('Département' for
            France and Region for other countries).
        admin2: A string correponding to an administrative code ( 'Département' number
            for France)
        postCode: A string corresponding to the ZIP code of location.
    """

    def __init__(self, raw_data: PlaceData) -> None:
        """Initialize a Place object.

        Args:
            raw_data: A dictionary representing the JSON response from 'places' REST API
                request. The structure is described by the PlaceData class.
        """
        self.raw_data = raw_data

    def __repr__(self) -> str:
        """Return string representation of this class.

        Returns:
            A string to represent the instance of the Place class using the name,
            country and admin area of the location.

            Example: <Place(name=Montréal, country=FR, admin=Languedoc-Roussillon)>
        """
        return "<{}(name={}, country={}, admin={})>".format(
            self.__class__.__name__, self.name, self.country, self.admin
        )

    def __str__(self) -> str:
        """Provide an easy way to identify the Place.

        Returns:
            A string to represent a Place instance with city name, Region name,
            department ID and the country name.

            For Examples:
                `Marseille - Provence-Alpes-Côte d'Azur (13) - FR`
                or `Montréal - Quebec - CA`
        """
        if self.country == "FR":
            return f"{self.name} - {self.admin} ({self.admin2}) - {self.country}"

        return f"{self.name} - {self.admin} - {self.country}"

    @property
    def insee(self) -> Optional[str]:
        """Return the INSEE ID of the place."""
        return self.raw_data.get("insee")

    @property
    def name(self) -> str:
        """Return the name of the place."""
        return self.raw_data["name"]

    @property
    def latitude(self) -> float:
        """Return the latitude of the place."""
        return self.raw_data["lat"]

    @property
    def longitude(self) -> float:
        """Return the longitude of the place."""
        return self.raw_data["lon"]

    @property
    def country(self) -> str:
        """Return the country code of the place."""
        return self.raw_data["country"]

    @property
    def admin(self) -> Optional[str]:
        """Return the admin of the place."""
        return self.raw_data.get("admin")

    @property
    def admin2(self) -> Optional[str]:
        """Return the admin2 of the place."""
        return self.raw_data.get("admin2")

    @property
    def postal_code(self) -> Optional[str]:
        """Return the postal code of the place."""
        return self.raw_data.get("postCode")



# -*- coding: utf-8 -*-
"""Rain in the next hour Python model for the Météo-France REST API."""
import sys
from datetime import datetime
from typing import Any
from typing import Dict
from typing import List
from typing import Optional

from meteofrance_api.helpers import timestamp_to_dateime_with_locale_tz

if sys.version_info >= (3, 8):
    from typing import TypedDict  # pylint: disable=no-name-in-module
else:
    from typing_extensions import TypedDict


class RainData(TypedDict):
    """Describing the data structure of rain object returned by the REST API."""

    position: Dict[str, Any]
    updated_on: int
    forecast: List[Dict[str, Any]]
    quality: int


class Rain:
    """Class to access the results of 'rain' REST API request.

    Attributes:
        position: A dictionary with metadata about the position of the forecast place.
        updated_on:  A timestamp as int corresponding to the latest update date.
        forecast: A list of dictionaries to describe the following next hour rain
            forecast.
        quality: An integer. Don't know yet the usage.
    """

    def __init__(self, raw_data: RainData) -> None:
        """Initialize a Rain object.

        Args:
            raw_data: A dictionary representing the JSON response from 'rain' REST API
                request. The structure is described by the RainData class.
        """
        self.raw_data = raw_data

    @property
    def position(self) -> Dict[str, Any]:
        """Return the position information of the rain forecast."""
        return self.raw_data["position"]

    @property
    def updated_on(self) -> int:
        """Return the update timestamp of the rain forecast."""
        return self.raw_data["updated_on"]

    @property
    def forecast(self) -> List[Dict[str, Any]]:
        """Return the rain forecast."""
        return self.raw_data["forecast"]

    @property
    def quality(self) -> int:
        """Return the quality of the rain forecast."""
        # TODO: don't know yet what is the usage
        return self.raw_data["quality"]

    def next_rain_date_locale(self) -> Optional[datetime]:
        """Estimate the date of the next rain in the Place timezone (Helper).

        Returns:
            A datetime instance representing the date estimation of the next rain within
            the next hour.
            If no rain is expected in the following hour 'None' is returned.

            The datetime use the location timezone.
        """
        # search first cadran with rain
        next_rain = next(
            (cadran for cadran in self.forecast if cadran["rain"] > 1), None
        )

        next_rain_dt_local: Optional[datetime] = None
        if next_rain is not None:
            # get the time stamp of the first cadran with rain
            next_rain_timestamp = next_rain["dt"]
            # convert timestamp in datetime with local timezone
            next_rain_dt_local = timestamp_to_dateime_with_locale_tz(
                next_rain_timestamp, self.position["timezone"]
            )

        return next_rain_dt_local

    def timestamp_to_locale_time(self, timestamp: int) -> datetime:
        """Convert timestamp in datetime with rain forecast location timezone (Helper).

        Args:
            timestamp: An integer representing the UNIX timestamp.

        Returns:
            A datetime instance corresponding to the timestamp with the timezone of the
                rain forecast location.
        """
        return timestamp_to_dateime_with_locale_tz(timestamp, self.position["timezone"])



# -*- coding: utf-8 -*-
"""Helpers to be used with the Météo-France REST API ."""
import math
from datetime import datetime
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple

from pytz import timezone
from pytz import utc

from .const import ALERT_COLOR_LIST_EN
from .const import ALERT_COLOR_LIST_FR
from .const import ALERT_TYPE_LIST_EN
from .const import ALERT_TYPE_LIST_FR
from .const import COASTAL_DEPARTMENT_LIST
from .const import VALID_DEPARTMENT_LIST
from .model.place import Place


def get_warning_text_status_from_indice_color(
    int_color: int, lang: str = "fr"
) -> Optional[str]:
    """Convert the color code (in int) in readable text (Helper).

    Args:
        int_color: Color status in int. Value expected between 1 and 4.
        lang: Optional; If language is equal 'fr' (default value) results will
            be in French. All other value will give results in English.

    Returns:
        Color status in text. French or English according to the lang parameter.
    """
    if lang == "fr":
        return ALERT_COLOR_LIST_FR[int_color]

    return ALERT_COLOR_LIST_EN[int_color]


def get_phenomenon_name_from_indice(
    int_phenomenon: int, lang: str = "fr"
) -> Optional[str]:
    """Convert the phenomenom code in readable text (Hepler).

    Args:
        int_phenomenon: ID of the phenomenom in int. Value expected between 1 and 9.
        lang: Optional; If language is equal "fr" (default value) results will
            be in French. All other value will give results in English.

    Returns:
        Phenomenom in text. French or English according to the lang parameter.
    """
    if lang == "fr":
        return ALERT_TYPE_LIST_FR[int_phenomenon]

    return ALERT_TYPE_LIST_EN[int_phenomenon]


def is_coastal_department(department_number: str) -> bool:
    """Identify when a second bulletin is availabe for coastal risks (Helper).

    Args:
        department_number: Department number on 2 characters

    Returns:
        True if the department have an additional coastal bulletin. False otherwise.
    """
    return department_number in COASTAL_DEPARTMENT_LIST


def is_valid_warning_department(department_number: str) -> bool:
    """Identify if there is a weather alert bulletin for this department (Helper).

    Weather alert buletins are available only for metropolitan France and Andorre.

    Args:
        department_number: Department number on 2 characters.

    Returns:
        True if a department is metropolitan France or Andorre.
    """
    return department_number in VALID_DEPARTMENT_LIST


def readeable_phenomenoms_dict(
    list_phenomenoms: List[Dict[str, int]], language: str = "fr"
) -> Dict[Optional[str], Optional[str]]:
    """Create a dictionary with human readable keys and values (Helper).

    Args:
        list_phenomenoms: Dictionary with phenomenon ID and color code of status.
        language: Optional; If language is equal "fr" (default value) results will
            be in French. All other value will give results in English.

    Returns:
        Dictionary with keys and value human readable.
    """
    # Init empty dictionary
    readable_dict = {}

    # Translate phenomenom name and alert level
    for phenomenom in list_phenomenoms:
        readable_dict[
            get_phenomenon_name_from_indice(phenomenom["phenomenon_id"], language)
        ] = get_warning_text_status_from_indice_color(
            phenomenom["phenomenon_max_color_id"], language
        )
    return readable_dict


def haversine(coord1: Tuple[float, float], coord2: Tuple[float, float]) -> float:
    """Compute distance in meters between to GPS coordinates using Harvesine formula.

    source: https://janakiev.com/blog/gps-points-distance-python/

    Args:
        coord1: Tuple with latitude and longitude in degrees for first point
        coord2: Tuple with latitude and longitude in degrees for second point

    Returns:
        Distance in meters between the two points
    """
    radius = 6372800  # Earth radius in meters
    lat1, lon1 = coord1
    lat2, lon2 = coord2
    phi1, phi2 = math.radians(lat1), math.radians(lat2)
    dphi = math.radians(lat2 - lat1)
    dlambda = math.radians(lon2 - lon1)

    a = (
        math.sin(dphi / 2) ** 2
        + math.cos(phi1) * math.cos(phi2) * math.sin(dlambda / 2) ** 2
    )

    return 2 * radius * math.atan2(math.sqrt(a), math.sqrt(1 - a))


def sort_places_versus_distance_from_coordinates(
    list_places: List[Place], gps_coord: Tuple[float, float]
) -> List[Place]:
    """Oder list of places according to the distance to a reference coordinates.

    Note: this helper is compensating the bad results of the API. Results in the API
    are generally sorted, but lot of cases identified where the order is inconsistent
    (example: Montréal)

    Args:
        list_places: List of Place instances to be ordered
        gps_coord: Tuple with latitude and longitude in degrees for the reference point

    Returns:
        List of Place instances ordered by distance to the reference point (nearest
            first)
    """
    sorted_places = sorted(
        list_places,
        key=lambda x: haversine((float(x.latitude), float(x.longitude)), gps_coord),
    )
    return sorted_places


def timestamp_to_dateime_with_locale_tz(timestamp: int, local_tz: str) -> datetime:
    """Convert timestamp in datetime (Helper).

    Args:
        timestamp: Timestamp.
        local_tz: Name of the timezone to be used to convert the timestamp.

    Returns:
        Datetime instance corresponding to the timestamp with a timezone.
    """
    # convert timestamp in datetime with UTC timezone
    dt_utc = utc.localize(datetime.utcfromtimestamp(timestamp))
    # convert datetime to local timezone
    return dt_utc.astimezone(timezone(local_tz))
